@@ -47,7 +47,10 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pSamples = INTEGER(rSamples);
-
+	if (!pSamples) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	dim = GET_DIM(rSamples);
 	numnodes = INTEGER(dim)[0];
 	numsamples = INTEGER(dim)[1];
@@ -59,7 +62,16 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
+	if (!pNodeNumCats) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
+	if (!pNodeCats) {
+		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	memset(pNodeCats, 0, numnodes*sizeof(int*));
 	memset(pNodeNumCats, 0, numnodes*sizeof(int));
 
@@ -98,10 +110,24 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 		if(maxCategories < pNodeNumCats[i])
 			maxCategories = pNodeNumCats[i];
 	}
-	//printf("maxCategories = %d\n", maxCategories);
+
 	pprobs = (int*)CATNET_MALLOC(maxCategories*maxCategories*sizeof(int));
+	if (!pprobs) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 
 	klmat = (double*)CATNET_MALLOC(numnodes*numnodes*sizeof(double));
+	if (!klmat) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		CATNET_FREE(pprobs);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
+	
 	memset(klmat, 0, numnodes*numnodes*sizeof(double));
 
 	psubSamples = 0;
@@ -115,7 +141,7 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 	for(nnode1 = 0; nnode1 < numnodes; nnode1++) {
 		psamples1 = pSamples;
 		numsamples1 = numsamples;
-		if(pPerturbations) {
+		if(pPerturbations && psubSamples) {
 			numsubsamples = 0;
 			for(j = 0; j < numsamples; j++) {
 				if(!pPerturbations[j * numnodes + nnode1]) {
@@ -126,7 +152,7 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 				numsamples1 = numsubsamples;
 			}
 		}
-		//printf("nnode = %d (%d)\n", nnode1, numsamples1);
+
 		for(nnode2 = 0; nnode2 < numnodes; nnode2++) {
 	
 			memset(pprobs, 0, maxCategories*maxCategories*sizeof(int));
@@ -144,7 +170,6 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 				if(fsum > 0) {
 					floglik -= fsum*(double)log((double)fsum);
 					floglik /= fsum;
-					//floglik *= numsamples;
 				}
 				klmat[nnode2*numnodes + nnode1] = -floglik;
 				continue;
@@ -170,7 +195,6 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 			}
 			if(fsum > 0) {
 				floglik /= fsum;
-				//floglik *= numsamples;
 			}
 			klmat[nnode2*numnodes + nnode1] = -floglik;
 		}
@@ -199,7 +223,8 @@ SEXP catnetEntropyPairwise(SEXP rSamples, SEXP rPerturbations) {
 	if(klmat) {
 		PROTECT(rvec = NEW_NUMERIC(numnodes*numnodes));
 		pvec = NUMERIC_POINTER(rvec);
-		memcpy(pvec, klmat, numnodes*numnodes*sizeof(double));
+		if (pvec && klmat)
+			memcpy(pvec, klmat, numnodes*numnodes*sizeof(double));
 		UNPROTECT(1);
 	}
 
@@ -231,7 +256,10 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pSamples = INTEGER(rSamples);
-
+	if (!pSamples) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	dim = GET_DIM(rSamples);
 	numnodes = INTEGER(dim)[0];
 	numsamples = INTEGER(dim)[1];
@@ -246,7 +274,16 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
+	if (!pNodeNumCats) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
+	if (!pNodeCats) {
+		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	memset(pNodeCats, 0, numnodes*sizeof(int*));
 	memset(pNodeNumCats, 0, numnodes*sizeof(int));
 
@@ -285,10 +322,24 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 		if(maxCategories < pNodeNumCats[i])
 			maxCategories = pNodeNumCats[i];
 	}
-	//printf("maxCategories = %d\n", maxCategories);
+
 	pprobs = (int*)CATNET_MALLOC(maxCategories*maxCategories*sizeof(int));
+	if (!pprobs) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 
 	klmat = (double*)CATNET_MALLOC(numnodes*numnodes*sizeof(double));
+	if (!klmat) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		CATNET_FREE(pprobs);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
+
 	memset(klmat, 0, numnodes*numnodes*sizeof(double));
 
 	psubSamples = 0;
@@ -302,7 +353,7 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 	for(nnode1 = 0; nnode1 < numnodes; nnode1++) {
 		psamples1 = pSamples;
 		numsamples1 = numsamples;
-		if(pPerturbations) {
+		if(pPerturbations && psubSamples) {
 			numsubsamples = 0;
 			for(j = 0; j < numsamples; j++) {
 				if(!pPerturbations[j * numnodes + nnode1]) {
@@ -313,7 +364,7 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 				numsamples1 = numsubsamples;
 			}
 		}
-		//printf("nnode = %d (%d)\n", nnode1, numsamples1);
+
 		for(nnode2 = 0; nnode2 < numnodes; nnode2++) {
 	
 			memset(pprobs, 0, maxCategories*maxCategories*sizeof(int));
@@ -331,7 +382,6 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 				if(fsum > 0) {
 					floglik -= fsum*(double)log((double)fsum);
 					floglik /= fsum;
-					//floglik *= numsamples;
 				}
 				klmat[nnode2*numnodes + nnode1] = -floglik;
 				continue;
@@ -357,7 +407,6 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 			}
 			if(fsum > 0) {
 				floglik /= fsum;
-				//floglik *= numsamples;
 			}
 			klmat[nnode2*numnodes + nnode1] = -floglik;
 		}
@@ -393,72 +442,18 @@ SEXP catnetEntropyOrder(SEXP rSamples, SEXP rPerturbations) {
 		pnoderanks[nnode1] = floglik;
 	}
 
-//printf("pnoderanks = ");
-//for(i = 0; i < numnodes; i++)
-//printf(" %f", pnoderanks[i]);
-//printf("\n");
 	_order<double>(pnoderanks, numnodes, porder, 1);
-//printf("porder = ");
-//for(i = 0; i < numnodes; i++)
-//printf(" %d", porder[i]+1);
-//printf("\n");
 
-	/*fsum = 0;
-	pnoderanks[porder[0]] = 0;
-	for(nnode1 = 1; nnode1 < numnodes; nnode1++) {
-		faux = 0;
-		for(i = 0; i < nnode1; i++)
-			if(faux < klmat[porder[i]*numnodes + porder[nnode1]])
-				faux = klmat[porder[i]*numnodes + porder[nnode1]];
-		pnoderanks[porder[nnode1]] = faux;
-printf("pnoderanks[%d] = %f\n", porder[nnode1], pnoderanks[porder[nnode1]]);
-		fsum += faux;
-	}
-
-	for(nnode1 = 0; nnode1 < numnodes; nnode1++) {
-		for(nnode2 = nnode1 + 1; nnode2 < numnodes; nnode2++) {
-			memcpy(pnoderanks + numnodes, pnoderanks, numnodes*sizeof(double));
-
-			for(d = nnode1; d <= nnode2; d++) {
-				faux = 0;
-				for(i = 0; i < d; i++) {
-					if(i == nnode1 && faux < klmat[porder[nnode2]*numnodes + porder[d]])
-						faux = klmat[porder[nnode2]*numnodes + porder[d]];
-					else if(i == nnode2 && faux < klmat[porder[nnode1]*numnodes + porder[d]])
-						faux = klmat[porder[nnode1]*numnodes + porder[d]];
-					else if(faux < klmat[porder[i]*numnodes + porder[d]])
-						faux = klmat[porder[i]*numnodes + porder[d]];
-				}
-				if(d == nnode1)
-					pnoderanks[numnodes+porder[nnode2]] = faux;
-				else if(d == nnode2)
-					pnoderanks[numnodes+porder[nnode1]] = faux;
-				else
-					pnoderanks[numnodes+porder[d]] = faux;
-			}
-			faux = 0;
-			for(i = 0; i < numnodes; i++)
-				faux += pnoderanks[numnodes+i];
-
-			if(faux > fsum) {
-printf("change %d <-> %d    %f (%f)\n", porder[nnode1]+1, porder[nnode2]+1, fsum, faux);
-				fsum = faux;
-				i = porder[nnode1];
-				porder[nnode1] = porder[nnode2];
-				porder[nnode2] = i;
-				memcpy(pnoderanks, pnoderanks + numnodes, numnodes*sizeof(double));
-			}
-		}
-	}*/
-
-	CATNET_FREE(klmat);
+	if (klmat)
+		CATNET_FREE(klmat);
 
 	for(i = 0; i < numnodes; i++)
 		porder[i]++;
 
 	PROTECT(rvec = NEW_INTEGER(numnodes));
 	pvec = INTEGER_POINTER(rvec);
-	memcpy(pvec, porder, numnodes*sizeof(int));
+	if (pvec && porder)
+		memcpy(pvec, porder, numnodes*sizeof(int));
 	UNPROTECT(1);
 
 	CATNET_FREE(porder);
@@ -489,6 +484,10 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pSamples = INTEGER(rSamples);
+	if (!pSamples) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 
 	dim = GET_DIM(rSamples);
 	numnodes = INTEGER(dim)[0];
@@ -498,7 +497,8 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 		UNPROTECT(1); //rSamples
 		PROTECT(rvec = NEW_NUMERIC(numnodes*numnodes));
 		pvec = NUMERIC_POINTER(rvec);
-		memset(pvec, 0, numnodes*numnodes*sizeof(double));
+		if (pvec)
+			memset(pvec, 0, numnodes*numnodes*sizeof(double));
 		UNPROTECT(1);
 		return rvec;
 	}
@@ -510,7 +510,16 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
+	if (!pNodeNumCats) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
+	if (!pNodeCats) {
+		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	memset(pNodeCats, 0, numnodes*sizeof(int*));
 	memset(pNodeNumCats, 0, numnodes*sizeof(int));
 
@@ -553,6 +562,17 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 	pprobs1 = (double*)CATNET_MALLOC(maxCategories*maxCategories*sizeof(double));
 	pprobs2 = (double*)CATNET_MALLOC(maxCategories*maxCategories*sizeof(double));
 
+	if (!pprobs1 || !pprobs2) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		if (pprobs1)
+			CATNET_FREE(pprobs1);
+		if (pprobs2)
+			CATNET_FREE(pprobs2);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
+
 	pSamplesPert = 0;
 	pPerturbations = 0;
 	if(!isNull(rPerturbations)) {
@@ -562,11 +582,20 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 	}
 
 	klmat = (double*)CATNET_MALLOC(numnodes*numnodes*sizeof(double));
+	if (!klmat) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		CATNET_FREE(pprobs1);
+		CATNET_FREE(pprobs2);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
+
 	memset(klmat, 0, numnodes*numnodes*sizeof(double));
 
 	for(nnode1 = 0; nnode1 < numnodes; nnode1++) {
 		numsamplesPert = 0;
-		if(pPerturbations) {
+		if(pPerturbations && pSamplesPert) {
 			for(j = 0; j < numsamples; j++) {
 				if(!pPerturbations[j * numnodes + nnode1]) {
 					memcpy(pSamplesPert + numsamplesPert*numnodes, pSamples + j*numnodes, numnodes*sizeof(int));
@@ -622,67 +651,6 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 				}
 			}
 			klmat[nnode2*numnodes + nnode1] += floglik;
-
-			/*memset(pprobs2, 0, maxCategories*maxCategories*sizeof(double));
-			// estimate logP(nnode2|nnode1) for the whole sample
-			for(j = 0; j < numsamples; j++) 
-				pprobs2[maxCategories*pSamples[j*numnodes + nnode1] + pSamples[j*numnodes + nnode2]]++;
-
-			memset(pprobs1, 0, maxCategories*maxCategories*sizeof(double));
-			// estimate logP(nnode2|nnode1) for the perturbed sub-sample only
-			for(j = 0; j < numsamplesPert; j++) 
-				pprobs1[maxCategories*pSamplesPert[j*numnodes + nnode1] + pSamplesPert[j*numnodes + nnode2]]++;
-
-			for(i = 0; i < pNodeNumCats[nnode1]; i++) {
-				fsum = 0;
-				for(j = 0; j < pNodeNumCats[nnode2]; j++)
-					fsum += pprobs1[maxCategories*i+j];
-				if(fsum <= 0)
-					continue;
-				faux = 1 / fsum;
-				for(j = 0; j < pNodeNumCats[nnode2]; j++)
-					pprobs1[maxCategories*i+j] *= faux;
-			}
-			for(i = 0; i < pNodeNumCats[nnode1]; i++) {
-				fsum = 0;
-				for(j = 0; j < pNodeNumCats[nnode2]; j++)
-					fsum += pprobs2[maxCategories*i+j];
-				if(fsum <= 0)
-					continue;
-				faux = 1 / fsum;
-				for(j = 0; j < pNodeNumCats[nnode2]; j++)
-					pprobs2[maxCategories*i+j] *= faux;
-			}
-
-			floglik = 0;
-			for(i = 0; i < pNodeNumCats[nnode1]; i++) {
-				for(j = 0; j < pNodeNumCats[nnode2]; j++) {
-					if(pprobs1[maxCategories*i+j] > 0 && pprobs2[maxCategories*i+j] > 0)
-						floglik += pprobs1[maxCategories*i+j]*
-						(double)log((double)pprobs1[maxCategories*i+j] / (double)pprobs2[maxCategories*i+j]);
-					else if(pprobs1[maxCategories*i+j] != 0 && pprobs2[maxCategories*i+j] == 0)
-						floglik = FLT_MAX;
-				}
-			}
-			klmat[nnode2*numnodes + nnode1] += floglik;*/
-
-			/*printf("nnode2 = %d: ", nnode2);
-			for(i = 0; i < pNodeNumCats[nnode2]; i++) {
-				printf(" [");
-				for(j = 0; j < pNodeNumCats[nnode1]; j++) {
-					printf("%f ", pprobs1[maxCategories*i+j]);
-				}
-				printf("] ");
-			}
-			printf("\n");
-			for(i = 0; i < pNodeNumCats[nnode2]; i++) {
-				printf(" [");
-				for(j = 0; j < pNodeNumCats[nnode1]; j++) {
-					printf("%f ", pprobs2[maxCategories*i+j]);
-				}
-				printf("] ");
-			}
-			printf(" KL = %f\n", floglik);*/
 		}
 	}
 
@@ -710,10 +678,12 @@ SEXP catnetKLpairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	PROTECT(rvec = NEW_NUMERIC(numnodes*numnodes));
 	pvec = NUMERIC_POINTER(rvec);
-	memcpy(pvec, klmat, numnodes*numnodes*sizeof(double));
+	if (pvec && klmat)
+		memcpy(pvec, klmat, numnodes*numnodes*sizeof(double));
 	UNPROTECT(1);
 
-	CATNET_FREE(klmat);
+	if (klmat)
+		CATNET_FREE(klmat);
 
 	//char str[128];
 	//sprintf(str, "Mem Balance  %d\n", (int)g_memcounter);
@@ -740,6 +710,10 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pSamples = INTEGER(rSamples);
+	if (!pSamples) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 
 	dim = GET_DIM(rSamples);
 	numnodes = INTEGER(dim)[0];
@@ -749,7 +723,8 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 		UNPROTECT(1); //rSamples
 		PROTECT(rvec = NEW_NUMERIC(numnodes*numnodes));
 		pvec = NUMERIC_POINTER(rvec);
-		memset(pvec, 0, numnodes*numnodes*sizeof(double));
+		if (pvec)
+			memset(pvec, 0, numnodes*numnodes*sizeof(double));
 		UNPROTECT(1);
 		return rvec;
 	}
@@ -761,7 +736,16 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
+	if (!pNodeNumCats) {
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
+	if (!pNodeCats) {
+		CATNET_FREE(pNodeNumCats);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 	memset(pNodeCats, 0, numnodes*sizeof(int*));
 	memset(pNodeNumCats, 0, numnodes*sizeof(int));
 
@@ -803,6 +787,16 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	pprobs1 = (double*)CATNET_MALLOC(maxCategories*maxCategories*sizeof(double));
 	pprobs2 = (double*)CATNET_MALLOC(maxCategories*maxCategories*sizeof(double));
+	if (!pprobs1 || !pprobs2) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		if (pprobs1)
+			CATNET_FREE(pprobs1);
+		if (pprobs2)
+			CATNET_FREE(pprobs2);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
 
 	pSamplesPert = 0;
 	pPerturbations = 0;
@@ -813,11 +807,20 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 	}
 
 	klmat = (double*)CATNET_MALLOC(numnodes*numnodes*sizeof(double));
+	if (!klmat) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		CATNET_FREE(pprobs1);
+		CATNET_FREE(pprobs2);
+		UNPROTECT(1); //rSamples
+		return rvec;
+	}
+	
 	memset(klmat, 0, numnodes*numnodes*sizeof(double));
 
 	for(nnode1 = 0; nnode1 < numnodes; nnode1++) {
 		numsamplesPert = 0;
-		if(pPerturbations) {
+		if(pPerturbations && pSamplesPert) {
 			for(j = 0; j < numsamples; j++) {
 				if(!pPerturbations[j * numnodes + nnode1]) {
 					memcpy(pSamplesPert + numsamplesPert*numnodes, pSamples + j*numnodes, numnodes*sizeof(int));
@@ -868,24 +871,6 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 				}
 			}
 			klmat[nnode2*numnodes + nnode1] += floglik;
-
-			/*printf("nnode2 = %d: ", nnode2);
-			for(i = 0; i < pNodeNumCats[nnode2]; i++) {
-				printf(" [");
-				for(j = 0; j < pNodeNumCats[nnode1]; j++) {
-					printf("%f ", pprobs1[maxCategories*i+j]);
-				}
-				printf("] ");
-			}
-			printf("\n");
-			for(i = 0; i < pNodeNumCats[nnode2]; i++) {
-				printf(" [");
-				for(j = 0; j < pNodeNumCats[nnode1]; j++) {
-					printf("%f ", pprobs2[maxCategories*i+j]);
-				}
-				printf("] ");
-			}
-			printf(" KL = %f\n", floglik);*/
 		}
 	}
 
@@ -913,10 +898,12 @@ SEXP catnetPearsonPairwise(SEXP rSamples, SEXP rPerturbations) {
 
 	PROTECT(rvec = NEW_NUMERIC(numnodes*numnodes));
 	pvec = NUMERIC_POINTER(rvec);
-	memcpy(pvec, klmat, numnodes*numnodes*sizeof(double));
+	if (pvec && klmat)
+		memcpy(pvec, klmat, numnodes*numnodes*sizeof(double));
 	UNPROTECT(1);
 
-	CATNET_FREE(klmat);
+	if (klmat)
+		CATNET_FREE(klmat);
 
 	//char str[128];
 	//sprintf(str, "Mem Balance  %d\n", (int)g_memcounter);
@@ -946,6 +933,10 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 
 	PROTECT(rSamples = AS_INTEGER(rSamples));
 	pRSamples = INTEGER(rSamples);
+	if (!pRSamples) {
+		UNPROTECT(1); //rSamples
+		return 0;
+	}
 
 	dim = GET_DIM(rSamples);
 	numnodes = INTEGER(dim)[0];
@@ -953,6 +944,10 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 
 	// pSamples are assumed positive indices
 	pSamples = (int*)CATNET_MALLOC(numnodes*numsamples*sizeof(int));
+	if (!pSamples) {
+		UNPROTECT(1); //rSamples
+		return 0;
+	}
 	memcpy(pSamples, pRSamples, numnodes*numsamples*sizeof(int));
 	UNPROTECT(1); //rSamples
 
@@ -962,7 +957,13 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 
 	// categoies
 	pNodeNumCats = (int*)CATNET_MALLOC(numnodes*sizeof(int));
+	if (!pNodeNumCats)
+		return 0;
 	pNodeCats = (int**)CATNET_MALLOC(numnodes*sizeof(int*));
+	if (!pNodeCats) {
+		CATNET_FREE(pNodeNumCats);
+		return 0;
+	}
 	memset(pNodeCats, 0, numnodes*sizeof(int*));
 	memset(pNodeNumCats, 0, numnodes*sizeof(int));
 
@@ -1003,6 +1004,25 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 	}
 
 	pprobs = (double*)CATNET_MALLOC(maxCategories*maxCategories*sizeof(double));
+	if (!pprobs) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		if(!isNull(rPerturbations))
+			UNPROTECT(1); //rPerturbations
+		return 0;
+	}
+
+	matPairs = (double*)CATNET_MALLOC(numnodes*numnodes*sizeof(double));
+	if (!matPairs) {
+		CATNET_FREE(pNodeNumCats);
+		CATNET_FREE(pNodeCats);
+		CATNET_FREE(pprobs);
+		if(!isNull(rPerturbations))
+			UNPROTECT(1); //rPerturbations
+		return 0;	
+	}
+
+	memset(matPairs, 0, numnodes*numnodes*sizeof(double));
 
 	pSamplesPert = 0;
 	pPerturbations = 0;
@@ -1012,12 +1032,9 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 		pSamplesPert = (int*)CATNET_MALLOC(numnodes*numsamples*sizeof(int));
 	}
 
-	matPairs = (double*)CATNET_MALLOC(numnodes*numnodes*sizeof(double));
-	memset(matPairs, 0, numnodes*numnodes*sizeof(double));
-
 	for(nnode1 = 0; nnode1 < numnodes; nnode1++) {
 		numsamplesPert = 0;
-		if(pPerturbations) {
+		if(pPerturbations && pSamplesPert) {
 			for(j = 0; j < numsamples; j++) {
 				if(!pPerturbations[j * numnodes + nnode1]) {
 					memcpy(pSamplesPert + numsamplesPert*numnodes, pSamples + j*numnodes, numnodes*sizeof(int));
@@ -1025,7 +1042,7 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 				}
 			}
 		}
-		//printf("\nnnode = %d (%d)\n", nnode1, numsamplesPert);
+
 		for(nnode2 = 0; nnode2 < numnodes; nnode2++) {
 	
 			if(nnode1 == nnode2)
@@ -1034,7 +1051,7 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 			ncount = 0;
 			memset(pprobs, 0, maxCategories*maxCategories*sizeof(double));
 			// estimate logP(nnode1|nnode2)
-			if(pPerturbations) {
+			if(pPerturbations && pSamplesPert) {
 				for(j = 0; j < numsamplesPert; j++) {
 					pprobs[maxCategories*pSamplesPert[j*numnodes + nnode2] + pSamplesPert[j*numnodes + nnode1]]++; 
 					ncount++;
@@ -1065,16 +1082,14 @@ double *catnetPairwiseCondLikelihood(SEXP rSamples, SEXP rPerturbations) {
 			matPairs[nnode1*numnodes + nnode2] = floglik;
 		}
 		fsum = 0; fmin = FLT_MAX; fmax = -FLT_MAX;
-//printf("matPairs[%d] = ", nnode1+1);
 		for(nnode2 = 0; nnode2 < numnodes; nnode2++) {
-//printf("%f ", matPairs[nnode1*numnodes + nnode2]);
 			fsum += matPairs[nnode1*numnodes + nnode2];
 			if(fmin > matPairs[nnode1*numnodes + nnode2])
 				fmin = matPairs[nnode1*numnodes + nnode2];
 			if(fmax < matPairs[nnode1*numnodes + nnode2])
 				fmax = matPairs[nnode1*numnodes + nnode2];
 		}
-//printf("\n");
+
 		fsum = 1;
 		if(fmax-fmin>0)
 			fsum = 1 / (fmax-fmin);
